@@ -24,7 +24,7 @@
 //! use openpgp::armor::{Reader, ReaderMode, Kind};
 //!
 //! let mut file = File::open("somefile.asc")?;
-//! let mut r = Reader::new(&mut file, ReaderMode::Tolerant(Some(Kind::File)));
+//! let mut r = Reader::from_reader(&mut file, ReaderMode::Tolerant(Some(Kind::File)));
 //! # Ok(()) }
 //! ```
 
@@ -558,8 +558,6 @@ pub struct Reader<'a> {
     kind: Option<Kind>,
     mode: ReaderMode,
     decode_buffer: Vec<u8>,
-    crc: Crc,
-    expect_crc: Option<u32>,
     initialized: bool,
     headers: Vec<(String, String)>,
     finalized: bool,
@@ -609,6 +607,21 @@ impl Default for CSFTransformer {
 impl<'a> Reader<'a> {
     /// Constructs a new filter for the given type of data.
     ///
+    /// This function is deprecated and will be removed in version
+    /// 2.0. Please use [`Reader::from_reader`][].
+    #[deprecated = "Use Reader::from_reader. `new` will be removed in version 2.0"]
+    pub fn new<R, M>(inner: R, mode: M) -> Self
+        where R: 'a + Read + Send + Sync,
+              M: Into<Option<ReaderMode>>
+    {
+        Self::from_buffered_reader(
+            Box::new(buffered_reader::Generic::with_cookie(inner, None,
+                                                           Default::default())),
+            mode, Default::default())
+    }
+
+    /// Constructs a new `Reader` from the given `io::Read`er.
+    ///
     /// [ASCII Armor], designed to protect OpenPGP data in transit,
     /// has been a source of problems if the armor structure is
     /// damaged.  For example, copying data manually from one program
@@ -638,7 +651,7 @@ impl<'a> Reader<'a> {
     /// let data = "yxJiAAAAAABIZWxsbyB3b3JsZCE="; // base64 over literal data packet
     ///
     /// let mut cursor = io::Cursor::new(&data);
-    /// let mut reader = Reader::new(&mut cursor, ReaderMode::VeryTolerant);
+    /// let mut reader = Reader::from_reader(&mut cursor, ReaderMode::VeryTolerant);
     ///
     /// let mut buf = Vec::new();
     /// reader.read_to_end(&mut buf)?;
@@ -666,7 +679,7 @@ impl<'a> Reader<'a> {
     ///      -----END PGP ARMORED FILE-----";
     ///
     /// let mut cursor = io::Cursor::new(&data);
-    /// let mut reader = Reader::new(&mut cursor, ReaderMode::Tolerant(Some(Kind::File)));
+    /// let mut reader = Reader::from_reader(&mut cursor, ReaderMode::Tolerant(Some(Kind::File)));
     ///
     /// let mut content = String::new();
     /// reader.read_to_string(&mut content)?;
@@ -675,17 +688,6 @@ impl<'a> Reader<'a> {
     /// # Ok(())
     /// # }
     /// ```
-    pub fn new<R, M>(inner: R, mode: M) -> Self
-        where R: 'a + Read + Send + Sync,
-              M: Into<Option<ReaderMode>>
-    {
-        Self::from_buffered_reader(
-            Box::new(buffered_reader::Generic::with_cookie(inner, None,
-                                                           Default::default())),
-            mode, Default::default())
-    }
-
-    /// Creates a `Reader` from an `io::Read`er.
     pub fn from_reader<R, M>(reader: R, mode: M) -> Self
         where R: 'a + Read + Send + Sync,
               M: Into<Option<ReaderMode>>
@@ -748,8 +750,6 @@ impl<'a> Reader<'a> {
             kind: None,
             mode,
             decode_buffer: Vec::<u8>::with_capacity(1024),
-            crc: Crc::new(),
-            expect_crc: None,
             headers: Vec::new(),
             initialized: false,
             finalized: false,
@@ -795,7 +795,7 @@ impl<'a> Reader<'a> {
     ///      -----END PGP ARMORED FILE-----";
     ///
     /// let mut cursor = io::Cursor::new(&data);
-    /// let mut reader = Reader::new(&mut cursor, ReaderMode::Tolerant(Some(Kind::File)));
+    /// let mut reader = Reader::from_reader(&mut cursor, ReaderMode::Tolerant(Some(Kind::File)));
     ///
     /// let mut content = String::new();
     /// reader.read_to_string(&mut content)?;
@@ -1174,8 +1174,8 @@ impl<'a> Reader<'a> {
             //     of data.
             //
             //     Note: this happens if the caller does `for c in
-            //     Reader::new(...).bytes() ...`.  Then it reads one
-            //     byte of decoded data at a time.
+            //     Reader::from_reader(...).bytes() ...`.  Then it
+            //     reads one byte of decoded data at a time.
             //
             //   - Small: if the caller only requests a few bytes at a
             //     time, we may as well double buffer to reduce
@@ -1232,8 +1232,6 @@ impl<'a> Reader<'a> {
                     &base64data, base64::STANDARD)
                     .map_err(|e| Error::new(ErrorKind::InvalidData, e))?;
 
-                self.crc.update(&self.decode_buffer);
-
                 let copied = cmp::min(buf.len(), self.decode_buffer.len());
                 buf[..copied].copy_from_slice(&self.decode_buffer[..copied]);
                 crate::vec_drain_prefix(&mut self.decode_buffer, copied);
@@ -1242,13 +1240,9 @@ impl<'a> Reader<'a> {
             } else {
                 // We can decode directly into the caller-supplied
                 // buffer.
-                let decoded = base64::decode_config_slice(
+                base64::decode_config_slice(
                     &base64data, base64::STANDARD, buf)
-                    .map_err(|e| Error::new(ErrorKind::InvalidData, e))?;
-
-                self.crc.update(&buf[..decoded]);
-
-                decoded
+                    .map_err(|e| Error::new(ErrorKind::InvalidData, e))?
             };
 
             self.prefix_remaining = prefix_remaining;
@@ -1281,20 +1275,6 @@ impl<'a> Reader<'a> {
                     && data[1..5].iter().all(is_base64_char)
                 {
                     /* Found.  */
-                    let crc = match base64::decode_config(
-                        &data[1..5], base64::STANDARD)
-                    {
-                        Ok(d) => d,
-                        Err(e) => return Err(Error::new(ErrorKind::InvalidInput, e)),
-                    };
-
-                    assert_eq!(crc.len(), 3);
-                    let crc =
-                        (crc[0] as u32) << 16
-                        | (crc[1] as u32) << 8
-                        | crc[2] as u32;
-
-                    self.expect_crc = Some(crc);
                     5
                 } else {
                     0
@@ -1333,13 +1313,6 @@ impl<'a> Reader<'a> {
                 }
             };
             self.source.consume(consumed);
-
-            if let Some(crc) = self.expect_crc {
-                if self.crc.finalize() != crc {
-                    return Err(Error::new(ErrorKind::InvalidInput,
-                                          "Bad CRC sum."));
-                }
-            }
         }
 
         Ok(decoded)
@@ -1451,14 +1424,14 @@ impl<'a> Reader<'a> {
 
                         // First, split off the line ending.
                         let crlf_line_end = line.ends_with(b"\r\n");
-                        line = &line[..line.len()
-                                         - if crlf_line_end { 2 } else { 1 }];
+                        line = &line[..line.len().saturating_sub(
+                            if crlf_line_end { 2 } else { 1 })];
 
                         // Now, trim whitespace off the line.
                         while Some(&b' ') == line.last()
                             || Some(&b'\t') == line.last()
                         {
-                            line = &line[..line.len() - 1];
+                            line = &line[..line.len().saturating_sub(1)];
                         }
 
                         text.extend_from_slice(line);
@@ -1899,7 +1872,7 @@ mod test {
                           LITERAL_NO_HEADER_WITH_CHKSUM_ASC[i],
                           LITERAL_NO_HEADER_ASC[i],
                           LITERAL_NO_NEWLINES_ASC[i]] {
-                let mut r = Reader::new(Cursor::new(test),
+                let mut r = Reader::from_reader(Cursor::new(test),
                                         ReaderMode::VeryTolerant);
                 let mut dearmored = Vec::<u8>::new();
                 r.read_to_end(&mut dearmored).unwrap();
@@ -1912,7 +1885,7 @@ mod test {
     #[test]
     fn dearmor_binary() {
         for bin in TEST_BIN.iter() {
-            let mut r = Reader::new(
+            let mut r = Reader::from_reader(
                 Cursor::new(bin), ReaderMode::Tolerant(Some(Kind::Message)));
             let mut buf = [0; 5];
             let e = r.read(&mut buf);
@@ -1922,7 +1895,7 @@ mod test {
 
     #[test]
     fn dearmor_wrong_kind() {
-        let mut r = Reader::new(
+        let mut r = Reader::from_reader(
             Cursor::new(&include_bytes!("../tests/data/armor/test-0.asc")[..]),
             ReaderMode::Tolerant(Some(Kind::Message)));
         let mut buf = [0; 5];
@@ -1932,18 +1905,23 @@ mod test {
 
     #[test]
     fn dearmor_wrong_crc() {
-        let mut r = Reader::new(
+        let mut r = Reader::from_reader(
             Cursor::new(
                 &include_bytes!("../tests/data/armor/test-0.bad-crc.asc")[..]),
             ReaderMode::Tolerant(Some(Kind::File)));
         let mut buf = [0; 5];
         let e = r.read(&mut buf);
-        assert!(e.is_err());
+        // Quoting RFC4880++:
+        //
+        // > An implementation MUST NOT reject an OpenPGP object when
+        // > the CRC24 footer is present, missing, malformed, or
+        // > disagrees with the computed CRC24 sum.
+        assert!(e.is_ok());
     }
 
     #[test]
     fn dearmor_wrong_footer() {
-        let mut r = Reader::new(
+        let mut r = Reader::from_reader(
             Cursor::new(
                 &include_bytes!("../tests/data/armor/test-2.bad-footer.asc")[..]
             ),
@@ -1962,7 +1940,7 @@ mod test {
 
     #[test]
     fn dearmor_no_crc() {
-        let mut r = Reader::new(
+        let mut r = Reader::from_reader(
             Cursor::new(
                 &include_bytes!("../tests/data/armor/test-1.no-crc.asc")[..]),
             ReaderMode::Tolerant(Some(Kind::File)));
@@ -1973,7 +1951,7 @@ mod test {
 
     #[test]
     fn dearmor_with_header() {
-        let mut r = Reader::new(
+        let mut r = Reader::from_reader(
             Cursor::new(
                 &include_bytes!("../tests/data/armor/test-3.with-headers.asc")[..]
             ),
@@ -1990,7 +1968,7 @@ mod test {
 
     #[test]
     fn dearmor_any() {
-        let mut r = Reader::new(
+        let mut r = Reader::from_reader(
             Cursor::new(
                 &include_bytes!("../tests/data/armor/test-3.with-headers.asc")[..]
             ),
@@ -2010,7 +1988,7 @@ mod test {
         // Slap some garbage in front and make sure it still reads ok.
         let mut b: Vec<u8> = "Some\ngarbage\nlines\n\t\r  ".into();
         b.extend_from_slice(armored);
-        let mut r = Reader::new(Cursor::new(b), ReaderMode::VeryTolerant);
+        let mut r = Reader::from_reader(Cursor::new(b), ReaderMode::VeryTolerant);
         let mut buf = [0; 5];
         let e = r.read(&mut buf);
         assert_eq!(r.kind(), Some(Kind::File));
@@ -2022,7 +2000,7 @@ mod test {
         // line of the header.
         let mut b: Vec<u8> = "Some\ngarbage\nlines\n\t.\r  ".into();
         b.extend_from_slice(armored);
-        let mut r = Reader::new(Cursor::new(b), ReaderMode::VeryTolerant);
+        let mut r = Reader::from_reader(Cursor::new(b), ReaderMode::VeryTolerant);
         let mut buf = [0; 5];
         let e = r.read(&mut buf);
         assert!(e.is_err());
@@ -2031,7 +2009,7 @@ mod test {
     #[test]
     fn dearmor() {
         for (bin, asc) in TEST_BIN.iter().zip(TEST_ASC.iter()) {
-            let mut r = Reader::new(
+            let mut r = Reader::from_reader(
                 Cursor::new(asc),
                 ReaderMode::Tolerant(Some(Kind::File)));
             let mut dearmored = Vec::<u8>::new();
@@ -2044,7 +2022,7 @@ mod test {
     #[test]
     fn dearmor_bytewise() {
         for (bin, asc) in TEST_BIN.iter().zip(TEST_ASC.iter()) {
-            let r = Reader::new(
+            let r = Reader::from_reader(
                 Cursor::new(asc),
                 ReaderMode::Tolerant(Some(Kind::File)));
             let mut dearmored = Vec::<u8>::new();
@@ -2059,12 +2037,12 @@ mod test {
     #[test]
     fn dearmor_yuge() {
         let yuge_key = crate::tests::key("yuge-key-so-yuge-the-yugest.asc");
-        let mut r = Reader::new(Cursor::new(yuge_key),
+        let mut r = Reader::from_reader(Cursor::new(yuge_key),
                                 ReaderMode::VeryTolerant);
         let mut dearmored = Vec::<u8>::new();
         r.read_to_end(&mut dearmored).unwrap();
 
-        let r = Reader::new(Cursor::new(yuge_key),
+        let r = Reader::from_reader(Cursor::new(yuge_key),
                             ReaderMode::VeryTolerant);
         let mut dearmored = Vec::<u8>::new();
         for c in r.bytes() {
@@ -2074,7 +2052,7 @@ mod test {
 
     #[test]
     fn dearmor_quoted() {
-        let mut r = Reader::new(
+        let mut r = Reader::from_reader(
             Cursor::new(
                 &include_bytes!("../tests/data/armor/test-3.with-headers-quoted.asc")[..]
             ),
@@ -2089,7 +2067,7 @@ mod test {
 
     #[test]
     fn dearmor_quoted_stripped() {
-        let mut r = Reader::new(
+        let mut r = Reader::from_reader(
             Cursor::new(
                 &include_bytes!("../tests/data/armor/test-3.with-headers-quoted-stripped.asc")[..]
             ),
@@ -2104,7 +2082,7 @@ mod test {
 
     #[test]
     fn dearmor_quoted_a_lot() {
-        let mut r = Reader::new(
+        let mut r = Reader::from_reader(
             Cursor::new(
                 &include_bytes!("../tests/data/armor/test-3.with-headers-quoted-a-lot.asc")[..]
             ),
@@ -2119,7 +2097,7 @@ mod test {
 
     #[test]
     fn dearmor_quoted_badly() {
-        let mut r = Reader::new(
+        let mut r = Reader::from_reader(
             Cursor::new(
                 &include_bytes!("../tests/data/armor/test-3.with-headers-quoted-badly.asc")[..]
             ),
@@ -2143,13 +2121,13 @@ mod test {
             let encoded = w.finalize().unwrap();
 
             let mut recovered = Vec::new();
-            Reader::new(Cursor::new(&encoded),
+            Reader::from_reader(Cursor::new(&encoded),
                         ReaderMode::Tolerant(Some(kind)))
                 .read_to_end(&mut recovered)
                 .unwrap();
 
             let mut recovered_any = Vec::new();
-            Reader::new(Cursor::new(&encoded), ReaderMode::VeryTolerant)
+            Reader::from_reader(Cursor::new(&encoded), ReaderMode::VeryTolerant)
                 .read_to_end(&mut recovered_any)
                 .unwrap();
 
