@@ -22,6 +22,40 @@ use crate::utils::{read_be_u64, write_be_u64};
 
 pub(crate) use crate::crypto::backend::ecdh::{encrypt, decrypt};
 
+/// Returns the default ECDH KDF hash function.
+pub(crate) fn default_ecdh_kdf_hash(curve: &Curve) -> HashAlgorithm {
+    match curve {
+        Curve::Cv25519 => HashAlgorithm::SHA256,
+        // From RFC6637:
+        Curve::NistP256 => HashAlgorithm::SHA256,
+        Curve::NistP384 => HashAlgorithm::SHA384,
+        Curve::NistP521 => HashAlgorithm::SHA512,
+        // Extrapolated from RFC6637:
+        Curve::BrainpoolP256 => HashAlgorithm::SHA256,
+        Curve::BrainpoolP512 => HashAlgorithm::SHA512,
+        // Conservative default.
+        Curve::Ed25519 // Odd: Not an encryption algo.
+            | Curve::Unknown(_) => HashAlgorithm::SHA512,
+    }
+}
+
+/// Returns the default ECDH KEK cipher.
+pub(crate) fn default_ecdh_kek_cipher(curve: &Curve) -> SymmetricAlgorithm {
+    match curve {
+        Curve::Cv25519 => SymmetricAlgorithm::AES128,
+        // From RFC6637:
+        Curve::NistP256 => SymmetricAlgorithm::AES128,
+        Curve::NistP384 => SymmetricAlgorithm::AES192,
+        Curve::NistP521 => SymmetricAlgorithm::AES256,
+        // Extrapolated from RFC6637:
+        Curve::BrainpoolP256 => SymmetricAlgorithm::AES128,
+        Curve::BrainpoolP512 => SymmetricAlgorithm::AES256,
+        // Conservative default.
+        Curve::Ed25519 // Odd: Not an encryption algo.
+            | Curve::Unknown(_) => SymmetricAlgorithm::AES256,
+    }
+}
+
 /// Wraps a session key.
 ///
 /// After using Elliptic-curve Diffie-Hellman to compute the shared
@@ -198,8 +232,6 @@ fn pkcs5_unpad(sk: Protected, target_len: usize) -> Result<Protected> {
 fn aes_key_wrap(algo: SymmetricAlgorithm, key: &Protected,
                 plaintext: &Protected)
                 -> Result<Vec<u8>> {
-    use crate::SymmetricAlgorithm::*;
-
     if plaintext.len() % 8 != 0 {
         return Err(Error::InvalidArgument(
             "Plaintext must be a multiple of 8".into()).into());
@@ -209,10 +241,7 @@ fn aes_key_wrap(algo: SymmetricAlgorithm, key: &Protected,
         return Err(Error::InvalidArgument("Bad key size".into()).into());
     }
 
-    let mut cipher = match algo {
-        AES128 | AES192 | AES256 => algo.make_encrypt_ecb(key)?,
-        _ => return Err(Error::UnsupportedSymmetricAlgorithm(algo).into()),
-    };
+    let mut cipher = algo.make_encrypt_ecb(key)?;
 
     //   Inputs:  Plaintext, n 64-bit values {P1, P2, ..., Pn}, and
     //            Key, K (the KEK).
@@ -273,8 +302,6 @@ fn aes_key_wrap(algo: SymmetricAlgorithm, key: &Protected,
 fn aes_key_unwrap(algo: SymmetricAlgorithm, key: &Protected,
                   ciphertext: &[u8])
                   -> Result<Protected> {
-    use crate::SymmetricAlgorithm::*;
-
     if ciphertext.len() % 8 != 0 {
         return Err(Error::InvalidArgument(
             "Ciphertext must be a multiple of 8".into()).into());
@@ -284,10 +311,7 @@ fn aes_key_unwrap(algo: SymmetricAlgorithm, key: &Protected,
         return Err(Error::InvalidArgument("Bad key size".into()).into());
     }
 
-    let mut cipher = match algo {
-        AES128 | AES192 | AES256 => algo.make_decrypt_ecb(key)?,
-        _ => return Err(Error::UnsupportedSymmetricAlgorithm(algo).into()),
-    };
+    let mut cipher = algo.make_decrypt_ecb(key)?;
 
     //   Inputs:  Ciphertext, (n+1) 64-bit values {C0, C1, ..., Cn}, and
     //            Key, K (the KEK).
