@@ -347,6 +347,28 @@ impl Curve {
             Unknown(_) => None,
         }
     }
+
+    /// Returns the curve's field size in bytes.
+    ///
+    /// # Examples
+    ///
+    /// ```rust
+    /// # fn main() -> sequoia_openpgp::Result<()> {
+    /// use sequoia_openpgp as openpgp;
+    /// use openpgp::types::Curve;
+    ///
+    /// assert_eq!(Curve::NistP256.field_size()?, 32);
+    /// assert_eq!(Curve::NistP384.field_size()?, 48);
+    /// assert_eq!(Curve::NistP521.field_size()?, 66);
+    /// assert_eq!(Curve::Ed25519.field_size()?, 32);
+    /// assert!(Curve::Unknown(Box::new([0x2B, 0x11])).field_size().is_err());
+    /// # Ok(()) }
+    /// ```
+    pub fn field_size(&self) -> Result<usize> {
+        self.bits()
+            .map(|bits| (bits + 7) / 8)
+            .ok_or_else(|| Error::UnsupportedEllipticCurve(self.clone()).into())
+    }
 }
 
 impl fmt::Display for Curve {
@@ -712,9 +734,7 @@ impl AEADAlgorithm {
     /// use sequoia_openpgp as openpgp;
     /// use openpgp::types::AEADAlgorithm;
     ///
-    /// assert!(AEADAlgorithm::EAX.is_supported());
-    ///
-    /// assert!(!AEADAlgorithm::OCB.is_supported());
+    /// assert!(! AEADAlgorithm::Private(100).is_supported());
     /// ```
     pub fn is_supported(&self) -> bool {
         self.is_supported_by_backend()
@@ -842,7 +862,6 @@ impl CompressionAlgorithm {
     /// use openpgp::types::CompressionAlgorithm;
     ///
     /// assert!(CompressionAlgorithm::Uncompressed.is_supported());
-    /// assert!(CompressionAlgorithm::Zip.is_supported());
     ///
     /// assert!(!CompressionAlgorithm::Private(101).is_supported());
     /// ```
